@@ -1,7 +1,7 @@
 /** @file str.cc
  * @brief Convert types to std::string
  */
-/* Copyright (C) 2009,2015 Olly Betts
+/* Copyright (C) 2009,2012,2015,2017 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,17 +27,18 @@
 #include <cstdio> // For snprintf() or sprintf().
 #include <cstdlib> // For abort().
 #include <string>
+#include <type_traits>
 
 using namespace std;
 
 // Much faster than snprintf() - also less generated code!
 template<class T>
-inline string
+static inline string
 tostring_unsigned(T value)
 {
-    STATIC_ASSERT_UNSIGNED_TYPE(T);
-    // We need a special case for 0, and we might as well handle all single
-    // digit numbers with it too.
+    static_assert(std::is_unsigned<T>::value, "Unsigned type required");
+    // Special case single digit positive numbers.
+    // FIXME: is this actually worthwhile?
     if (value < 10) return string(1, '0' + char(value));
     char buf[(sizeof(T) * 5 + 1) / 2];
     char * p = buf + sizeof(buf);
@@ -51,11 +52,11 @@ tostring_unsigned(T value)
 }
 
 template<class T>
-inline string
+static inline string
 tostring(T value)
 {
-    // We need a special case for 0, and we might as well handle all single
-    // digit positive numbers with it too.
+    // Special case single digit positive numbers.
+    // FIXME: is this actually worthwhile?
     if (value < 10 && value >= 0) return string(1, '0' + char(value));
 
     bool negative = (value < 0);
@@ -117,7 +118,7 @@ str(unsigned long long value)
 }
 
 template<class T>
-inline string
+static inline string
 format(const char * fmt, T value)
 {
     char buf[128];
