@@ -313,48 +313,67 @@ parse_pdfinfo_field(const char * p, const char * end, string & out, const char *
     parse_pdfinfo_field((P), (END), (OUT), FIELD":", CONST_STRLEN(FIELD) + 1)
 
 static void
-get_pdf_metainfo(const string & file, string &author, string &title,
+parse_pdf_metainfo(const string& pdfinfo, string &author, string &title,
+		   string &keywords, string &topic, int& pages)
+{
+    const char * p = pdfinfo.data();
+    const char * end = p + pdfinfo.size();
+    while (p != end) {
+	const char * start = p;
+	p = static_cast<const char *>(memchr(p, '\n', end - p));
+	const char * eol;
+	if (p) {
+	    eol = p;
+	    ++p;
+	} else {
+	    p = eol = end;
+	}
+	switch (*start) {
+	    case 'A':
+		PARSE_PDFINFO_FIELD(start, eol, author, "Author");
+		break;
+	    case 'K':
+		PARSE_PDFINFO_FIELD(start, eol, keywords, "Keywords");
+		break;
+	    case 'P': {
+		string s;
+		PARSE_PDFINFO_FIELD(start, eol, s, "Pages");
+		if (!s.empty())
+		    pages = atoi(s.c_str());
+		break;
+	    }
+	    case 'S':
+		PARSE_PDFINFO_FIELD(start, eol, topic, "Subject");
+		break;
+	    case 'T':
+		PARSE_PDFINFO_FIELD(start, eol, title, "Title");
+		break;
+	}
+    }
+}
+
+static void
+get_pdf_metainfo(int fd, string &author, string &title,
+		 string &keywords, string &topic, int& pages)
+{
+    try {
+	string pdfinfo;
+	run_filter(fd, "pdfinfo -enc UTF-8 -", false, &pdfinfo);
+	parse_pdf_metainfo(pdfinfo, author, title, keywords, topic, pages);
+    } catch (ReadError) {
+	// It's probably best to index the document even if pdfinfo fails.
+    }
+}
+
+static void
+get_pdf_metainfo(const string& file, string &author, string &title,
 		 string &keywords, string &topic, int& pages)
 {
     try {
 	string cmd = "pdfinfo -enc UTF-8";
 	append_filename_argument(cmd, file);
-	string pdfinfo = stdout_to_string(cmd, false);
-
-	const char * p = pdfinfo.data();
-	const char * end = p + pdfinfo.size();
-	while (p != end) {
-	    const char * start = p;
-	    p = static_cast<const char *>(memchr(p, '\n', end - p));
-	    const char * eol;
-	    if (p) {
-		eol = p;
-		++p;
-	    } else {
-		p = eol = end;
-	    }
-	    switch (*start) {
-		case 'A':
-		    PARSE_PDFINFO_FIELD(start, eol, author, "Author");
-		    break;
-		case 'K':
-		    PARSE_PDFINFO_FIELD(start, eol, keywords, "Keywords");
-		    break;
-		case 'P': {
-		    string s;
-		    PARSE_PDFINFO_FIELD(start, eol, s, "Pages");
-		    if (!s.empty())
-			pages = atoi(s.c_str());
-		    break;
-		}
-		case 'S':
-		    PARSE_PDFINFO_FIELD(start, eol, topic, "Subject");
-		    break;
-		case 'T':
-		    PARSE_PDFINFO_FIELD(start, eol, title, "Title");
-		    break;
-	    }
-	}
+	parse_pdf_metainfo(stdout_to_string(cmd, false),
+			   author, title, keywords, topic, pages);
     } catch (ReadError) {
 	// It's probably best to index the document even if pdfinfo fails.
     }
@@ -570,7 +589,8 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 	if (cmd_it != commands.end()) {
 	    // Easy "run a command and read text or HTML from stdout or a
 	    // temporary file" cases.
-	    string cmd = cmd_it->second.cmd;
+	    auto& filter = cmd_it->second;
+	    string cmd = filter.cmd;
 	    if (cmd.empty()) {
 		skip(urlterm, context, "required filter not installed",
 		     d.get_size(), d.get_mtime(), SKIP_VERBOSE_ONLY);
@@ -586,7 +606,7 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 		     SKIP_VERBOSE_ONLY);
 		return;
 	    }
-	    bool use_shell = cmd_it->second.use_shell();
+	    bool use_shell = filter.use_shell();
 	    bool substituted = false;
 	    string tmpout;
 	    size_t pcent = 0;
@@ -616,9 +636,9 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 			    // Use a temporary file with a suitable extension
 			    // in case the command cares, and for more helpful
 			    // error messages from the command.
-			    if (cmd_it->second.output_type == "text/html") {
+			    if (filter.output_type == "text/html") {
 				tmpout = get_tmpfile("tmp.html");
-			    } else if (cmd_it->second.output_type == "image/svg+xml") {
+			    } else if (filter.output_type == "image/svg+xml") {
 				tmpout = get_tmpfile("tmp.svg");
 			    } else {
 				tmpout = get_tmpfile("tmp.txt");
@@ -649,7 +669,7 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 	    try {
 		if (!tmpout.empty()) {
 		    // Output in temporary file.
-		    (void)stdout_to_string(cmd, use_shell);
+		    run_filter(cmd, use_shell);
 		    if (!load_file(tmpout, dump, NOCACHE)) {
 			throw ReadError("Couldn't read output file");
 		    }
@@ -659,10 +679,10 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 		    // filing system.
 		} else {
 		    // Output on stdout.
-		    dump = stdout_to_string(cmd, use_shell);
+		    run_filter(cmd, use_shell, &dump);
 		}
-		const string & charset = cmd_it->second.output_charset;
-		if (cmd_it->second.output_type == "text/html") {
+		const string & charset = filter.output_charset;
+		if (filter.output_type == "text/html") {
 		    MyHtmlParser p;
 		    p.ignore_metarobots();
 		    p.description_as_sample = description_as_sample;
@@ -685,7 +705,7 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 		    sample = p.sample;
 		    author = p.author;
 		    created = p.created;
-		} else if (cmd_it->second.output_type == "image/svg+xml") {
+		} else if (filter.output_type == "image/svg+xml") {
 		    SvgParser svgparser;
 		    svgparser.parse(dump);
 		    dump = svgparser.dump;
@@ -749,17 +769,15 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 		// FIXME: What charset is the file?  Look at contents?
 	    }
 	} else if (mimetype == "application/pdf") {
-	    string cmd = "pdftotext -enc UTF-8";
-	    append_filename_argument(cmd, file);
-	    cmd += " -";
+	    const char* cmd = "pdftotext -enc UTF-8 - -";
 	    try {
-		dump = stdout_to_string(cmd, false);
+		run_filter(d.get_fd(), cmd, false, &dump);
 	    } catch (ReadError) {
 		skip_cmd_failed(urlterm, context, cmd,
 				d.get_size(), d.get_mtime());
 		return;
 	    }
-	    get_pdf_metainfo(file, author, title, keywords, topic, pages);
+	    get_pdf_metainfo(d.get_fd(), author, title, keywords, topic, pages);
 	} else if (mimetype == "application/postscript") {
 	    // There simply doesn't seem to be a Unicode capable PostScript to
 	    // text converter (e.g. pstotext always outputs ISO-8859-1).  The
@@ -782,11 +800,11 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 	    append_filename_argument(cmd, file);
 	    append_filename_argument(cmd, tmpfile);
 	    try {
-		(void)stdout_to_string(cmd, false);
+		run_filter(cmd, false);
 		cmd = "pdftotext -enc UTF-8";
 		append_filename_argument(cmd, tmpfile);
 		cmd += " -";
-		dump = stdout_to_string(cmd, false);
+		run_filter(cmd, false, &dump);
 	    } catch (ReadError) {
 		skip_cmd_failed(urlterm, context, cmd,
 				d.get_size(), d.get_mtime());
@@ -797,7 +815,8 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 		throw;
 	    }
 	    try {
-		get_pdf_metainfo(tmpfile, author, title, keywords, topic, pages);
+		get_pdf_metainfo(tmpfile, author, title, keywords, topic,
+				 pages);
 	    } catch (...) {
 		unlink(tmpfile.c_str());
 		throw;
@@ -924,7 +943,7 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 	    cmd += " 'Documents/1/Pages/*.fpage'";
 	    try {
 		XpsXmlParser xpsparser;
-		dump = stdout_to_string(cmd, false);
+		run_filter(cmd, false, &dump);
 		// Look for Byte-Order Mark (BOM).
 		if (startswith(dump, "\xfe\xff") || startswith(dump, "\xff\xfe")) {
 		    // UTF-16 in big-endian/little-endian order - we just
@@ -973,10 +992,9 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 	    author = svgparser.author;
 	} else if (mimetype == "application/vnd.debian.binary-package" ||
 		   mimetype == "application/x-debian-package") {
-	    string cmd("dpkg-deb -f");
-	    append_filename_argument(cmd, file);
-	    cmd += " Description";
-	    const string & desc = stdout_to_string(cmd, false);
+	    const char* cmd = "dpkg-deb -f - Description";
+	    string desc;
+	    run_filter(d.get_fd(), cmd, false, &desc);
 	    // First line is short description, which we use as the title.
 	    string::size_type idx = desc.find('\n');
 	    title.assign(desc, 0, idx);
@@ -987,7 +1005,8 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 		   mimetype == "application/x-rpm") {
 	    string cmd("rpm -q --qf '%{SUMMARY}\\n%{DESCRIPTION}' -p");
 	    append_filename_argument(cmd, file);
-	    const string & desc = stdout_to_string(cmd, false);
+	    string desc;
+	    run_filter(cmd, false, &desc);
 	    // First line is summary, which we use as the title.
 	    string::size_type idx = desc.find('\n');
 	    title.assign(desc, 0, idx);
