@@ -1,7 +1,7 @@
 /** @file runfilter.cc
  * @brief Run an external filter and capture its output in a std::string.
  */
-/* Copyright (C) 2003,2006,2007,2009,2010,2011,2013,2015,2017 Olly Betts
+/* Copyright (C) 2003,2006,2007,2009,2010,2011,2013,2015,2017,2018 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,6 +58,8 @@
 
 using namespace std;
 
+static int devnull = -1;
+
 #if defined HAVE_FORK && defined HAVE_SOCKETPAIR
 bool
 command_needs_shell(const char * p)
@@ -154,8 +156,8 @@ handle_signal(int signum)
 
 }
 
-void
-runfilter_init()
+static inline void
+runfilter_init_signal_handlers_()
 {
     struct sigaction sa;
     sa.sa_handler = handle_signal;
@@ -203,8 +205,8 @@ handle_signal(int signum)
 
 }
 
-void
-runfilter_init()
+static inline void
+runfilter_init_signal_handlers_()
 {
     old_hup_handler = signal(SIGHUP, handle_signal);
     old_int_handler = signal(SIGINT, handle_signal);
@@ -221,16 +223,34 @@ command_needs_shell(const char *)
     return true;
 }
 
-void
-runfilter_init()
+static inline void
+runfilter_init_signal_handlers_()
 {
 }
 #endif
 
-string
-stdout_to_string(const string &cmd, bool use_shell, int alt_status)
+void
+runfilter_init()
+{
+    runfilter_init_signal_handlers_();
+    devnull = open("/dev/null", O_WRONLY);
+    if (devnull < 0) {
+	cerr << "Failed to open /dev/null: " << strerror(errno) << endl;
+	exit(1);
+    }
+    // Ensure that devnull isn't fd 0, 1 or 2 (stdin, stdout or stderr) and
+    // that we have open fds for stdin, stdout and stderr.  This simplifies the
+    // code after fork() because it doesn't need to worry about such corner
+    // cases.
+    while (devnull <= 2) {
+	devnull = dup(devnull);
+    }
+}
+
+void
+run_filter(int fd_in, const string& cmd, bool use_shell, string* out,
+	   int alt_status)
 {
-    string out;
 #if defined HAVE_FORK && defined HAVE_SOCKETPAIR
     // We want to be able to get the exit status of the child process.
     signal(SIGCHLD, SIG_DFL);
@@ -255,6 +275,12 @@ stdout_to_string(const string &cmd, bool use_shell, int alt_status)
 	// Close the parent's side of the socket pair.
 	close(fds[0]);
 
+	if (fd_in != -1) {
+	    // Connect piped input to stdin.
+	    dup2(fd_in, 0);
+	    close(fd_in);
+	}
+
 	// Connect stdout to our side of the socket pair.
 	dup2(fds[1], 1);
 
@@ -333,15 +359,11 @@ use_shell_after_all:
 	    if (!quoted) {
 		// Handle simple cases of redirection.
 		if (strcmp(word, ">/dev/null") == 0) {
-		    int fd = open(word + 1, O_WRONLY);
-		    if (fd != -1 && fd != 1) dup2(fd, 1);
-		    close(fd);
+		    dup2(devnull, 1);
 		    continue;
 		}
 		if (strcmp(word, "2>/dev/null") == 0) {
-		    int fd = open(word + 2, O_WRONLY);
-		    if (fd != -1 && fd != 2) dup2(fd, 2);
-		    close(fd);
+		    dup2(devnull, 2);
 		    continue;
 		}
 		if (strcmp(word, "2>&1") == 0) {
@@ -428,7 +450,7 @@ use_shell_after_all:
 	    pid_to_kill_on_signal = 0;
 	    throw ReadError(status);
 	}
-	out.append(buf, res);
+	if (out) out->append(buf, res);
     }
 
     close(fd);
@@ -452,7 +474,7 @@ use_shell_after_all:
 	    (void)pclose(fh);
 	    throw ReadError("fread failed");
 	}
-	out.append(buf, len);
+	if (out) out->append(buf, len);
     }
     int status = pclose(fh);
 #endif
@@ -460,7 +482,7 @@ use_shell_after_all:
     if (WIFEXITED(status)) {
 	int exit_status = WEXITSTATUS(status);
 	if (exit_status == 0 || exit_status == alt_status)
-	    return out;
+	    return;
 	if (exit_status == 127)
 	    throw NoSuchFilter();
     }
