@@ -52,6 +52,10 @@
 #include "utils.h"
 #include "values.h"
 
+#ifndef HAVE_STRPTIME
+#include "portability/strptime.h"
+#endif
+
 #include "gnu_getopt.h"
 
 using namespace std;
@@ -754,6 +758,12 @@ badhex:
 		value = Xapian::Unicode::tolower(value);
 		break;
 	    case Action::LOAD: {
+		// If there's no input, just issue a warning.
+		if (value.empty()) {
+		    report_location(DIAG_WARN, fname, line_no);
+		    cerr << "Empty filename in LOAD action" << endl;
+		    break;
+		}
 		bool truncated = false;
 		string filename = std::move(value);
 		// FIXME: Use NOATIME if we own the file or are root.
@@ -967,6 +977,9 @@ badhex:
 		break;
 	    }
 	    case Action::DATE: {
+		// Do nothing for empty input.
+		if (value.empty()) break;
+
 		const string & type = action.get_string_arg();
 		string yyyymmdd;
 		if (type == "unix") {
@@ -976,9 +989,10 @@ badhex:
 		    int m = tm->tm_mon + 1;
 		    yyyymmdd = date_to_string(y, m, tm->tm_mday);
 		} else if (type == "yyyymmdd") {
-		    if (value.length() == 8) yyyymmdd = value;
+		    if (value.length() != 8) break;
+		    yyyymmdd = value;
 		}
-		if (yyyymmdd.empty()) break;
+
 		// Date (YYYYMMDD)
 		doc.add_boolean_term("D" + yyyymmdd);
 		yyyymmdd.resize(6);
@@ -1119,6 +1133,27 @@ index_file(const char *fname, istream &stream,
     database.commit();
 }
 
+static void
+show_help(int exit_code)
+{
+    cout << PROG_NAME " - " PROG_DESC "\n"
+"Usage: " PROG_NAME " [OPTIONS] DATABASE INDEXER_SCRIPT [INPUT_FILE]...\n"
+"\n"
+"Creates or updates a Xapian database with the data from the input files listed\n"
+"on the command line.  If no files are specified, data is read from stdin.\n"
+"\n"
+"See https://xapian.org/docs/omega/scriptindex.html for documentation of the\n"
+"format for INDEXER_SCRIPT.\n"
+"\n"
+"Options:\n"
+"  -v, --verbose       display additional messages to aid debugging\n"
+"      --overwrite     create the database anew (the default is to update if\n"
+"                      the database already exists)\n";
+    print_stemmer_help("");
+    print_help_and_version_help("");
+    exit(exit_code);
+}
+
 int
 main(int argc, char **argv)
 try {
@@ -1138,16 +1173,15 @@ try {
 	{ 0, 0, NULL, 0 }
     };
 
-    bool more = true, show_help = false;
-    while (more) {
-	switch (gnu_getopt_long(argc, argv, "vs:hV", longopts, NULL)) {
-	    case EOF:
-		more = false;
-		break;
+    int getopt_ret;
+    while ((getopt_ret = gnu_getopt_long(argc, argv, "vs:hV",
+					 longopts, NULL)) != -1) {
+	switch (getopt_ret) {
 	    default:
+		show_help(1);
+		break;
 	    case 'h': // --help
-		show_help = true;
-		more = false;
+		show_help(0);
 		break;
 	    case 'V': // --version
 		print_package_info(PROG_NAME);
@@ -1173,23 +1207,8 @@ try {
 
     argv += optind;
     argc -= optind;
-    if (show_help || argc < 2) {
-	cout << PROG_NAME " - " PROG_DESC "\n"
-"Usage: " PROG_NAME " [OPTIONS] DATABASE INDEXER_SCRIPT [INPUT_FILE]...\n"
-"\n"
-"Creates or updates a Xapian database with the data from the input files listed\n"
-"on the command line.  If no files are specified, data is read from stdin.\n"
-"\n"
-"See https://xapian.org/docs/omega/scriptindex.html for documentation of the\n"
-"format for INDEXER_SCRIPT.\n"
-"\n"
-"Options:\n"
-"  -v, --verbose       display additional messages to aid debugging\n"
-"      --overwrite     create the database anew (the default is to update if\n"
-"                      the database already exists)\n";
-	print_stemmer_help("");
-	print_help_and_version_help("");
-	exit(show_help ? 0 : 1);
+    if (argc < 2) {
+	show_help(1);
     }
 
     parse_index_script(argv[1]);
