@@ -25,7 +25,7 @@
 #include <cerrno>
 #include <string>
 
-#ifdef USE_ICONV
+#ifdef HAVE_ICONV
 # include <iconv.h>
 #endif
 #include <xapian.h>
@@ -191,11 +191,11 @@ convert_to_utf8(string & text, const string & charset)
 
     if (false) {
 try_iconv:
-#ifdef USE_ICONV
+#ifdef HAVE_ICONV
 	iconv_t conv = iconv_open("UTF-8", charset.c_str());
 	if (conv == reinterpret_cast<iconv_t>(-1))
 	    return;
-	ICONV_INPUT_TYPE in = const_cast<char *>(text.c_str());
+	ICONV_CONST char* in = const_cast<char *>(text.c_str());
 	size_t in_len = text.size();
 	while (in_len) {
 	    char * out = buf;
